package com.xydl.cac.controller;

import com.xydl.cac.entity.IcdConfigType;
import com.xydl.cac.entity.IcdConfigTypeAtt;
import com.xydl.cac.exception.BusinessException;
import com.xydl.cac.model.ColumnModel;
import com.xydl.cac.model.IcdAttUpdateModel;
import com.xydl.cac.model.Response;
import com.xydl.cac.service.DataService;
import com.xydl.cac.service.IcdFileConfigService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.nio.charset.Charset;
import java.util.List;

@RestController
@Api(tags = {"IcdConfig相关接口"})
@RequestMapping("icdconfig")
@Slf4j
public class IcdConfigController extends BasicController {

    @Resource
    IcdFileConfigService configService;
    @Resource
    DataService dataService;

    @PostMapping("upload")
    @ApiOperation("处理icd文件")
    public Response<String> upload(@RequestParam("file") MultipartFile file) throws Exception {
        if (file != null && file.getInputStream() != null) {
            String content = IOUtils.toString(file.getInputStream(), Charset.forName("UTF-8"));
            configService.upload(content);
            return Response.success("OK");
        } else {
            return Response.fail("缺少上传文件");
        }
    }

    @GetMapping("list")
    @ApiOperation("查询全部类型列表")
    public Response<List<IcdConfigType>> list(String iedName) throws Exception {
        List<IcdConfigType> result = configService.list(iedName);
        return Response.success(result);
    }

    @GetMapping("iedList")
    @ApiOperation("查询IED列表")
    public Response<List<String>> iedList() {
        List<String> result = configService.iedList();
        return Response.success(result);
    }

    @PostMapping("update")
    @ApiOperation("更新ICD类型配置")
    public Response<String> update(@RequestBody IcdConfigType item) throws Exception {
        configService.update(item);
        return Response.success("OK");
    }

    @PostMapping("updateAtt")
    @ApiOperation("更新ICD类型属性配置")
    public Response<String> updateAtt(@RequestBody IcdAttUpdateModel model) throws Exception {
        configService.updateAtt(model);
        return Response.success("OK");
    }

    @PostMapping("delete")
    @ApiOperation("删除ICD类型配置")
    public Response<String> delete(@Validated @NotNull(message = "id不能为空!") Integer id) throws Exception {
        if (id == null) {
            throw new BusinessException("id不能为空!");
        }
        configService.delete(id);
        return Response.success("OK");
    }

    @PostMapping("deleteAtt")
    @ApiOperation("删除ICD类型属性配置")
    public Response<String> deleteAtt(@Validated @NotNull(message = "attid不能为空!") Integer attid) throws Exception {
        if (attid == null) {
            throw new BusinessException("attid不能为空!");
        }
        configService.deleteAtt(attid);
        return Response.success("OK");
    }

    @GetMapping("tableList")
    @ApiOperation("查询data表名")
    public Response<List<ColumnModel>> tableList() throws Exception {
        List<ColumnModel> result = dataService.getDataTables();
        return Response.success(result);
    }

    @GetMapping("colList")
    @ApiOperation("查询data表字段名")
    public Response<List<ColumnModel>> colList(String tableName) throws Exception {
        List<ColumnModel> result = dataService.getDataTableColumns(tableName);
        return Response.success(result);
    }

    @PostMapping("clearAll")
    @ApiOperation("清空所有配置")
    public Response<String> clearAll() throws Exception {
        configService.clearAll();
        return Response.success("OK");
    }

    @GetMapping("compare61850")
    @ApiOperation("比对61850的不同点")
    public Response<List<String>> compare61850() throws Exception {
        List<String> result = configService.compare61850();
        return Response.success(result);
    }

}
