package com.xydl.cac.service.impl;

import com.xydl.cac.entity.*;
import com.xydl.cac.model.ColumnModel;
import com.xydl.cac.model.ConditionModel;
import com.xydl.cac.model.OnePage;
import com.xydl.cac.repository.ModevTypeRepository;
import com.xydl.cac.service.DataService;
import com.xydl.cac.util.DataTable;
import com.xydl.cac.util.DataUtil;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.BeanPropertyRowMapper;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.ParseException;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class DataServiceImpl implements DataService {

    private String schema;

    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    ModevTypeRepository modevTypeRepository;

    @Override
    public List<ColumnModel> getDataTables() throws Exception {
        List<ModevType> typeList = modevTypeRepository.findAll();
        List<ColumnModel> tables = new ArrayList<>();
        String sql = "SHOW TABLES";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        for (String name : list) {
            if (name.startsWith("data_")) {
                ColumnModel item = new ColumnModel();
                item.setName(name);
                for (ModevType type : typeList) {
                    if (name.equals(type.getTablename())) {
                        item.setComment(type.getMc());
                        break;
                    }
                }
                tables.add(item);
            }
        }
        return tables;
    }

    @Override
    public List<ColumnModel> getDataTableColumns(String tableName) throws Exception {
        if (StringUtils.isBlank(schema)) {
            this.getDatabase();
        }

        List<ColumnModel> result = new ArrayList<>();
        String sql = "SELECT COLUMN_NAME name, COLUMN_COMMENT comment FROM information_schema.columns WHERE TABLE_NAME='"
                + tableName + "' AND TABLE_SCHEMA='" + schema + "' ORDER BY name,comment";
        List<ColumnModel> list = jdbcTemplate.query(sql, new BeanPropertyRowMapper<>(ColumnModel.class));
        for (ColumnModel item : list) {
            String col = item.getName();
            if (!col.equals("id") && !col.equals("eqmid")
                    && !col.equals("acquisitionTime") && !col.equals("d_time")
                    && !col.equals("create_time") && !col.equals("update_time")
                    && !col.equals("isupload")) {
                result.add(item);
            }
        }
        return result;
    }

    private void getDatabase() {
        String sql = "select database()";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        schema = list.get(0);
    }


    @Override
    public OnePage<Map<String, Object>> getData(NSensor sensor, List<ModevTypePoint> points, ConditionModel model) throws Exception {
        if (model.getNumPerDay() != null && model.getNumPerDay() > 0) {
            OnePage<Map<String, Object>> page;
            long totalDays = 0;
            // 需要抽取数据
            if (model.getPageNum() != null || model.getPageSize() != null) {

                // 当前一页
                int dayPerPage = model.getPageSize() / model.getNumPerDay();
                if (dayPerPage < 1) {
                    dayPerPage = 1;
                }
                ConditionModel newmodel = new ConditionModel();

                // 处理开始和结束时间
                Date end = model.getEndTime();
                if (end == null) {
                    end = new Date();
                }
                end = DateUtil.addDay(end, 1);
                end = DateUtil.getDayZero(end);
                if (model.getStartTime() != null) {
                    totalDays = DateUtil.getDifferenceInDays(model.getStartTime(), end);
                }

                if (model.getPageNum() > 1) {
                    int p = model.getPageNum() - 1;
                    end = DateUtil.addDay(end, -dayPerPage * p);
                }
                newmodel.setEndTime(end);
                Date start = DateUtil.addDay(end, -dayPerPage);
                if (model.getStartTime() == null || start.compareTo(model.getStartTime()) > 0) {
                    newmodel.setStartTime(start);
                } else {
                    newmodel.setStartTime(model.getStartTime());
                }

                page = this.doSqlData(sensor, points, newmodel);
                page.setTotal(totalDays * model.getNumPerDay());

            } else {
                // 全量
                page = this.doSqlData(sensor, points, model);
            }

            // 从数据里每天抽取num条
            List<Map<String, Object>> content = DataUtil.pickNumPerDay(page.getContent(), model.getNumPerDay());
            page.setContent(content);

            page.setPageNum(model.getPageNum());
            page.setPageSize(content.size());
            return page;
        } else {
            // 不抽取数据
            OnePage<Map<String, Object>> page = this.doSqlData(sensor, points, model);
            return page;
        }
    }

    private OnePage<Map<String, Object>> doSqlData(NSensor sensor, List<ModevTypePoint> points, ConditionModel model) {
        String devField = DataTable.getDevidField(sensor.getTableName());
        String timeField = DataTable.getTimeField(sensor.getTableName());
        OnePage result = new OnePage();

        String sqlFrom = " FROM " + sensor.getTableName();
        String sqlWhere = " WHERE " + devField + "=" + sensor.getDevId();
        if (model.getStartTime() != null) {
            sqlWhere = sqlWhere + " AND " + timeField + " >= '" + DateUtil.format(model.getStartTime()) + "'";
        }
        if (model.getEndTime() != null) {
            sqlWhere = sqlWhere + " AND " + timeField + " < '" + DateUtil.format(model.getEndTime()) + "'";
        }
        String sqlOrder = " ORDER BY " + timeField + " DESC";
        String sqlLimit = "";
        if (model.getPageNum() != null && model.getPageSize() != null) {
            result.setPageNum(model.getPageNum());
            result.setPageSize(model.getPageSize());
            sqlLimit = String.format(" LIMIT %d,%d", (model.getPageNum() - 1) * model.getPageSize(), model.getPageSize());
        }

        String sqlSelect = "SELECT DATE_FORMAT(" + timeField + ", '%Y-%m-%d %H:%i:%s') as acquisitionTime";
        for (ModevTypePoint point : points) {
            if (StringUtils.isNotBlank(point.getField())) {
                sqlSelect = sqlSelect + ", " + point.getField();
            }
        }

        String sqlCount = "SELECT count(*)" + sqlFrom + sqlWhere;
        String sql = sqlSelect + sqlFrom + sqlWhere + sqlOrder + sqlLimit;

        Long count = jdbcTemplate.queryForObject(sqlCount, Long.class);
        List<Map<String, Object>> content = jdbcTemplate.queryForList(sql);

        result.setTotal(count);
        result.setContent(content);
        return result;
    }

    @Override
    public void clearAllBind() {
        String sql = "UPDATE n_sensor SET icd_id=null";
        jdbcTemplate.execute(sql);
    }
}
