package com.xydl.cac.service.impl;

import com.xydl.cac.entity.*;
import com.xydl.cac.exception.BusinessException;
import com.xydl.cac.model.IcdAttUpdateModel;
import com.xydl.cac.repository.*;
import com.xydl.cac.service.IcdFileConfigService;
import com.xydl.cac.util.IcdXmlUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.*;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class IcdFileConfigServiceImpl implements IcdFileConfigService {

    @Resource
    IcdConfigTypeRepository repository;
    @Resource
    IcdConfigTypeAttRepository attRepository;
    @Resource
    IcdConfigTypeInstRepository instRepository;
    @Resource
    private JdbcTemplate jdbcTemplate;
    @Resource
    RptparamindexRepository rptparamindexRepository;

    @Override
    public void upload(String xml) throws Exception {
        LinkedHashMap<String, IcdConfigType> result = IcdXmlUtil.loadIcdType(xml);
        Collection<IcdConfigType> configList = result.values();
        for (IcdConfigType config : configList) {
            List<IcdConfigType> list = repository.findByIedNameAndLdeviceInstAndLnClass(
                    config.getIedName(), config.getLdeviceInst(), config.getLnClass());
            if (CollectionUtils.isEmpty(list)) {
                repository.save(config);
            } else {
                config.setId(list.get(0).getId());
            }

            attRepository.deleteByIcdConfigTypeId(config.getId());
            if (config.getAttMap() != null) {
                Collection<IcdConfigTypeAtt> attList = config.getAttMap().values();
                for (IcdConfigTypeAtt item : attList) {
                    item.setIcdConfigTypeId(config.getId());
                    item.buildInsts();
                }
                attRepository.saveAll(attList);
            }

            instRepository.deleteByIcdConfigTypeId(config.getId());
            if (config.getInstMap() != null) {
                Collection<IcdConfigTypeInst> instList = config.getInstMap().values();
                for (IcdConfigTypeInst item : instList) {
                    item.setIcdConfigTypeId(config.getId());
                }
                instRepository.saveAll(instList);
            }
        }
    }

    @Override
    public List<String> iedList() {
        String sql = "SELECT DISTINCT ied_name FROM icd_config_type";
        List<String> list = jdbcTemplate.queryForList(sql, String.class);
        return list;
    }

    @Override
    public List<IcdConfigType> list(String iedName) throws Exception {
        List<IcdConfigType> result;
        if (StringUtils.isNotBlank(iedName)) {
            result = repository.findByIedName(iedName);
        } else {
            result = repository.findAll();
        }
        for (IcdConfigType item : result) {
            List<IcdConfigTypeAtt> atts = attRepository.findByIcdConfigTypeId(item.getId());
            item.setAttList(atts);
            List<IcdConfigTypeInst> insts = instRepository.findByIcdConfigTypeId(item.getId());
            item.setInstList(insts);
            for (IcdConfigTypeInst inst : insts) {
                String param = item.getIedName() + item.getLdeviceInst() + "/" + item.getLnClass()
                        + inst.getInst();
                inst.setParamIndex(param);
            }
        }
        return result;
    }

    @Override
    public void update(IcdConfigType item) throws Exception {
        Optional<IcdConfigType> optional = repository.findById(item.getId());
        if (!optional.isPresent()) {
            throw new BusinessException("未找到该项");
        }
        IcdConfigType r = optional.get();
        if (r.getTableName() != null && !r.getTableName().equals(item.getTableName())) {
            List<IcdConfigTypeAtt> attList = attRepository.findByIcdConfigTypeId(r.getId());
            for (IcdConfigTypeAtt att : attList) {
                att.setColName(null);
            }
            attRepository.saveAll(attList);
        }
        r.setTableName(item.getTableName());
        repository.save(r);
    }

    @Override
    public void updateAtt(IcdAttUpdateModel model) throws Exception {
        if (CollectionUtils.isEmpty(model.getIdList())) {
            throw new BusinessException("id列表不能为空");
        }
        for (Integer id : model.getIdList()) {
            Optional<IcdConfigTypeAtt> optional = attRepository.findById(id);
            if (!optional.isPresent()) {
                throw new BusinessException("未找到该项" + id);
            }
            IcdConfigTypeAtt r = optional.get();
            r.setColName(model.getColName());
            attRepository.save(r);
        }
    }

    @Override
    public void delete(Integer id) {
        instRepository.deleteByIcdConfigTypeId(id);
        attRepository.deleteByIcdConfigTypeId(id);
        repository.deleteById(id);
    }

    @Override
    public void deleteAtt(Integer attid) {
        attRepository.deleteById(attid);
    }

    @Override
    public void clearAll() {
        instRepository.deleteAll();
        attRepository.deleteAll();
        repository.deleteAll();
    }

    @Override
    public List<String> compare61850() {
        List<String> param618List = new ArrayList<>();
        List<Rptparamindex> rptList = rptparamindexRepository.findAll();
        for (Rptparamindex rpt : rptList) {
            param618List.add(rpt.getParamindex());
        }

        List<String> paramList = new ArrayList<>();
        List<IcdConfigType> typeLIst = repository.findAll();
        for (IcdConfigType type : typeLIst) {
            List<IcdConfigTypeInst> instList = instRepository.findByIcdConfigTypeId(type.getId());
            List<IcdConfigTypeAtt> attList = attRepository.findByIcdConfigTypeId(type.getId());
            for (IcdConfigTypeInst inst : instList) {
                String param = type.getIedName() + type.getLdeviceInst() + "/" + type.getLnClass()
                        + inst.getInst();
                for (IcdConfigTypeAtt att : attList) {
                    if (att.containInst(inst.getInst())) {
                        String paramindex = param + "$" + att.getParam();
                        paramList.add(paramindex);
                    }
                }
            }
        }

        List<String> result = new ArrayList<>();
        for (String str : param618List) {
            if (!paramList.contains(str)) {
                result.add("少了" + str);
            }
        }
        for (String str : paramList) {
            if (!param618List.contains(str)) {
                result.add("多了" + str);
            }
        }
        return result;
    }

}
