package com.xydl.cac.service.impl;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelWriter;
import com.alibaba.excel.write.metadata.WriteSheet;
import com.alibaba.excel.write.style.column.LongestMatchColumnWidthStyleStrategy;
import com.xydl.cac.entity.ModevTypePoint;
import com.xydl.cac.excel.CacExcelListener;
import com.xydl.cac.excel.CacLine;
import com.xydl.cac.model.SensorDetail;
import com.xydl.cac.repository.ModevTypePointRepository;
import com.xydl.cac.service.ModevTypeService;
import com.xydl.cac.service.N103SensorService;
import com.xydl.cac.service.ReportService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class ReportServiceImpl implements ReportService {

    @Resource
    N103SensorService n103SensorService;
    @Resource
    ModevTypeService modevTypeService;
    @Resource
    ModevTypePointRepository typePointRepository;

    @Override
    public void exportSensor(SensorDetail<Map<String, Object>> detail, OutputStream output) throws Exception {
        List<List<String>> heads = new ArrayList<>();
        List<String> head = new ArrayList<>();
        head.add("时间");
        heads.add(head);
        for (ModevTypePoint point : detail.getPoints()) {
            head = new ArrayList<>();
            head.add(point.getFieldDesc());
            heads.add(head);
        }

        List<List<String>> list = new ArrayList<>();
        for (Map<String, Object> map : detail.getContent()) {
            List<String> line = new ArrayList<>();
            Object data = map.get("acquisitionTime");
            line.add(String.valueOf(data));
            for (ModevTypePoint point : detail.getPoints()) {
                data = map.get(point.getField());
                if (data == null) {
                    data = "";
                }
                line.add(String.valueOf(data));
            }
            list.add(line);
        }

        // 导出Excel
        ExcelWriter excelWriter = EasyExcel.write(output).inMemory(true)
                .registerWriteHandler(new LongestMatchColumnWidthStyleStrategy()).build();

        WriteSheet writeSheet0 = EasyExcel.writerSheet(0, detail.getSensor().getName())
                .head(heads)
                .build();
        excelWriter.write(list, writeSheet0);

        excelWriter.finish();
    }

    @Override
    public void importCac(InputStream input) throws Exception {
        CacExcelListener listener = new CacExcelListener(n103SensorService, typePointRepository,
                modevTypeService.listAll());
        EasyExcel.read(input, CacLine.class, listener)
                .sheet(0)
                .doRead();
    }
}
