package com.xydl.cac.service.impl;

import com.xydl.cac.entity.Warning;
import com.xydl.cac.model.ConditionModel;
import com.xydl.cac.repository.WarningRepository;
import com.xydl.cac.service.WarningService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.persistence.criteria.Predicate;

@Service
@Slf4j
@Transactional(rollbackFor = Exception.class)
public class WarningServiceImpl implements WarningService {

    @Resource
    WarningRepository repository;

    @Override
    public Page<Warning> list(ConditionModel condition) throws Exception {
        PageRequest request = PageRequest.of(condition.getPageNum(), condition.getPageSize());
        Specification<Warning> specification = (root, query, builder) -> {
            Predicate predicate = builder.conjunction();
            if (condition.getDevId() != null) {
                predicate.getExpressions().add(builder.equal(root.get("eqmid"), condition.getDevId().toString()));
            }
            if (StringUtils.isNotBlank(condition.getState())) {
                predicate.getExpressions().add(builder.equal(root.get("state"), condition.getState()));
            }
            if (condition.getStartTime() != null) {
                predicate.getExpressions().add(builder.greaterThan(root.get("warnTime"), condition.getStartTime()));
            }
            if (condition.getEndTime() != null) {
                predicate.getExpressions().add(builder.lessThanOrEqualTo(root.get("warnTime"), condition.getEndTime()));
            }
            query.orderBy(builder.desc(root.get("warnTime")));
            return predicate;
        };
        Page<Warning> result = repository.findAll(specification, request);
        return result;
    }
}
